--[[ Begin LUA DataModel core script init code

	This library depends upon certain global objects to be present.
 	Those objects are as follows:
		modelCore - This is the Lua Console that handles connections between
				this script, and the application that the script is running in
		
		resumed_thread - The name of the suspended coroutine that is to be resumed on a resume event.
		
		dataModelType - The name of the dataModelType that the current dataModel core is connected to. This is for
		preferences support and such.

--]]

-- Global vars that control various things, settable from the preferences
trace = false
debug = false
info = false

restart_lua_on_dead_coroutine_call = false

-- Retrieve preferences from the preferences engine here.

running_thread = ""
trace_notify_task = false


-- Waiting threads table
waiting_threads = {}

-- Cache for pre-serialized requests (like for loading data remotely)
request_cache = {}

-- Cache for pre-built argument lists
argument_cache = {}

-- System settings table.
system_settings = {}

-- The target object that performs async operations
async_target = {}

-- target table ( for script-targeted objects )
targets = {}

-- Service timeout list
timeouts = {}

-- Last access time for each service, can be used for multiple types of RPC services
serviceLastAccess = {}

-- Variable to determine if debug status messages are sent or not.
debugStatus = false

-- Resume a yielded thread
resume = as3.toas3( function( event )
	local threadTaskID = as3.tolua( event.taskID )
	local err = as3.tolua( event.errState )

	if trace then printTrace( "Lua::Resuming thread with ID " .. threadTaskID) end
	
	if waiting_threads[ threadTaskID ] ~= nil then
		local thread = waiting_threads[ threadTaskID ]
		if thread then
			if coroutine.status( thread ) ~= "dead" then
				running_thread = threadTaskID
				assert(coroutine.resume( thread, event.eventData, err ))
	 		end
		end
	end
end)

restart = function()
	-- statusMsg("ERROR", "Lua Error: Lua attempted to resume a coroutine that was dead.")
	if restart_lua_on_dead_coroutine_call == true then
		sendMVCMessage("restartLuaEngine", nil, nil)
	end
end

-- TODO: Modify this to support a logging level. All the support structure code (SerializableObject, DynamicElement, etc. )
-- don't need to be flooding the logs with a million messages unless the log level is set to a certain level.
printTrace = function( message )
	if message ~= nil then
		-- print( message )
		message = "LUA-TASK::" .. running_thread .. "::" .. tostring(message)
		modelCore.logMessage( as3.toas3( message ))
	else
		print("Log message was nil")
	end
 
end

-- Confirm that an object is an as3 value or not, and if not, convert it
castAs3 = function( target )
	if as3.isas3value( target ) == false then
		return as3.toas3( target )
	else
		return target
	end
end



waitForResponse = function()
	coroutine.yield()
end


showAlertPrompt = function( message, settings ) 
	-- Display a popup message 
	as3.class.mx.controls.Alert.show( as3.toas3(message), as3.toas3("Lua Script Alert:"))
end

askYesOrNo = function( message ) 
	-- Show an alert box with a yes/no question, and pause execution.
	modelCore.askYesOrNo( as3.toas3( running_thread ), as3.toas3( message ))
	local result = coroutine.yield()

	return as3.tolua( result )
end


promptForArguments = function(message, argumentList ) 
	if not as3.isas3value( message ) then
		message = as3.toas3( message )
	end
	if not as3.isas3value( argumentList ) then
		argumentList = as3.toarray( argumentList )
	end
	args = {}
	args[ "prompt" ] = message
	args["__taskID__"] = running_thread
	args = as3.toobject( args )
	
	modelCore.promptForArguments( as3.toas3( running_thread ), message, argumentList)
	
	local results, err = coroutine.yield()
	results = As3Object:new( results )
	return results, err
end

sleep = function( timeout ) 
	printTrace("going to sleep")
	modelCore.sleep( as3.toas3( running_thread ), as3.toas3( timeout ))
	coroutine.yield()
end


-- ##################### Initialization Logic goes here ###################

-- Add an event listener to the script ui (known as "modelCore" in the Lua global environment)
-- to allow operations to notify the script that it needs to resume.
modelCore.addEventListener( "resume", resume, false, 0, true )

printTrace("libLUA: LUA system library initialized.")

-- End LSA DataModel core script init code